<?php

namespace FiloBlu\Refilo\Model\Exporter;

use FiloBlu\Refilo\Model\Adapter\Mongo;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Framework\App\ResourceConnection;
use Magento\Store\Model\StoreManagerInterface;
use stdClass;

/**
 * Class Storelocator
 * @package FiloBlu\Refilo\Model\Exporter
 */
class Storelocator extends AbstractExporter
{
    /**
     * @var ResourceConnection
     */
    private $resourceConnection;

    /**
     * @var StoreManagerInterface
     */
    private $storeManager;

    /**
     * Countries constructor.
     * @param ResourceConnection $resourceConnection
     * @param ScopeConfigInterface $scopeConfig
     * @param Mongo $mongo
     * @param StoreManagerInterface $storeManager
     */
    public function __construct(
        ResourceConnection $resourceConnection,
        ScopeConfigInterface $scopeConfig,
        Mongo $mongo,
        StoreManagerInterface $storeManager
    )
    {
        parent::__construct($mongo, $scopeConfig);
        $this->resourceConnection = $resourceConnection;
        $this->storeManager = $storeManager;
    }

    public function preExecute()
    {
    }

    public function getCollections(): iterable
    {
        yield 'storelocator';
    }

    /**
     * @throws \MongoDB\Driver\Exception\Exception
     */
    public function execute()
    {
        $bulkData = [];

        foreach ($this->storeManager->getStores() as $key => $value) {

            if (!$value->getIsActive()) {
                continue;
            }

            $storeview_id = $value->getId();
            $where = "enabled = 1 AND (store_ids like '$storeview_id,%' OR store_ids like ',$storeview_id%' OR store_ids like '%,$storeview_id,%' OR store_ids = '0' OR store_ids = '$storeview_id')";

            $queryStores = "SELECT *,
                            CONCAT('/filoblu/storelocator/stores/image',fss.marker_image) as marker_image_url,
                            CONCAT('/filoblu/storelocator/stores/image',fss.image) as image_url FROM filoblu_storelocator_stores as fss
                            WHERE $where;";

            $queryTags = "SELECT fsst.store_id, fst.*, CONCAT('/filoblu/storelocator/stores/image',fst.image) as image_url FROM filoblu_storelocator_store_tag as fsst
                          INNER JOIN filoblu_storelocator_tags as fst on fsst.tag_id = fst.tag_id";
            $queryImages = "SELECT fssi.store_id, fsi.*, CONCAT('/filoblu/storelocator/stores/image',fsi.image) as image_url FROM filoblu_storelocator_store_image as fssi
                            INNER JOIN filoblu_storelocator_images as fsi on fssi.image_id = fsi.image_id";
            $queryWorkingTimes = "SELECT * FROM filoblu_storelocator_store_workingtimes;";

            $resultStores = $this->resourceConnection->getConnection()->fetchAll($queryStores);
            $resultTags = $this->resourceConnection->getConnection()->fetchAll($queryTags);
            $resultImages = $this->resourceConnection->getConnection()->fetchAll($queryImages);
            $resultWorkingTimes = $this->resourceConnection->getConnection()->fetchAll($queryWorkingTimes);

            foreach ($resultStores as $key => $store) {

                $store_id = $store['store_id'];
                unset($store['store_id']);
                $parser_store = (object)$store;
                $phone_1 = $parser_store->phone_1;
                $phone_2 = $parser_store->phone_2;
                $parser_store = json_decode(json_encode((object)$store, JSON_NUMERIC_CHECK));
                $parser_store->_id = $store_id;
                $parser_store->is_enabled_for_pickup_in_store = boolval($parser_store->is_enabled_for_pickup_in_store);
                $neededTags = array_filter(
                    $resultTags,
                    function ($e) use (&$store_id) {
                        return $e['store_id'] == $store_id;
                    }
                );

                $neededTags = json_decode(json_encode(array_values($neededTags), JSON_NUMERIC_CHECK));

                $parser_store->tags = new \StdClass();
                $parser_store->tags = $neededTags;

                $neededImages = array_filter(
                    $resultImages,
                    function ($e) use (&$store_id) {
                        return $e['store_id'] == $store_id;
                    }
                );
                $neededImages = json_decode(json_encode(array_values($neededImages), JSON_NUMERIC_CHECK | JSON_PRESERVE_ZERO_FRACTION));

                $parser_store->images = new \StdClass();
                $parser_store->images = $neededImages;

                $neededWorkingTimes = array_filter(
                    $resultWorkingTimes,
                    function ($e) use (&$store_id) {
                        return $e['store_id'] == $store_id;
                    }
                );
                $neededWorkingTimes = json_decode(json_encode(array_values($neededWorkingTimes), JSON_NUMERIC_CHECK));

                $parser_store->working_times = new \StdClass();
                $parser_store->working_times = $neededWorkingTimes;

                $parser_store->_id = "{$store_id}_{$storeview_id}";
                $parser_store->store_code = $value->getCode();
                $parser_store->code = $store['store_code'];

                $parser_store->phone_1 = $phone_1;
                $parser_store->phone_2 = $phone_2;
                
                $bulkData[] = $parser_store;
            }

        }

        try {
            $this->getConnection()->flushCollection('storelocator', $this->getDatabase());
            $this->getConnection()->update(
                '_id',
                $bulkData,
                'storelocator',
                true,
                $this->getDatabase()
            );
        } catch (\MongoDB\Driver\Exception\Exception $e) {
            echo $e->getMessage();
        }
    }
}
